/*
	CsoundKRateCmd.cc

	Will Eighteen, 2000: will@opaline.demon.co.uk             
*/

#include "CsoundKRateCmd.h"
#include "Application.h"
#include <LineEdit.h>
#include <MessageDialogManager.h>
#include "OpResources.h"


CsoundKRateCmd::CsoundKRateCmd ( const char *name, int active ) :  Cmd ( name, active )
{
	_popupRealised = False;
}


void CsoundKRateCmd::doCommand()
{
	if ( _popupRealised == True )
		return;

	char buf[30];
	char help[100];
	
	_edit = new LineEdit ( ( char * ) name(), TRUE,
									&CsoundKRateCmd::enterCallback,
									&CsoundKRateCmd::cancelCallback,
									( void * ) this );	
                
	/* retrieve resource to be displayed */
 	snprintf ( buf, sizeof ( buf ),"%u", mixwaveResources->kRate() );
	buf [ sizeof ( buf ) - 1 ] = 0;
	_edit->setColumns ( strlen ( buf ) );	// set width before instantiation
	snprintf ( help, sizeof ( help ), "Specifies csound control rate (time \
resolution)." );
	help [ sizeof ( help ) - 1 ] = 0;
	_edit->setHelp ( help );
	_edit->execute();
	_edit->setText ( buf );	// can't set text before instantiation...
	_popupRealised = True;
}


void CsoundKRateCmd::undoCommand()
{
	// not implemented
}


void CsoundKRateCmd::enterCallback ( void *clientData, char *text )
{
	float num;
	CsoundKRateCmd *obj = ( CsoundKRateCmd * ) clientData;

	sscanf ( text, "%f", &num );

	if ( ( num > 0 ) && ( num <= ( unsigned  int ) mixwaveResources->samplesPerSec() ) )
		obj->enter ( num );	

	obj->_popupRealised = False;
	delete obj->_edit;
}


/* specify resource to be set on 'enter' or 'ok' */
void CsoundKRateCmd::enter ( float kr )
{
	float ksmps = mixwaveResources->samplesPerSec() / kr;

	if ( ( ksmps != ( unsigned int ) ksmps ) || ( kr != ( unsigned int ) kr ) )
	{
		char msg[256];
		snprintf ( msg, sizeof ( msg ), "sampleRate/kRate = %ld/%f = %f;\nshould be an integer. \
Change kRate to an integer\ndivisor of sampleRate.", mixwaveResources->samplesPerSec(),
					kr, ksmps );
		msg [ sizeof ( msg ) - 1 ] = 0;
		messageDialogManager->post ( msg );
	}
	else
		mixwaveResources->kRate ( ( unsigned int ) kr );
}


void CsoundKRateCmd::cancelCallback ( void *clientData )
{
	CsoundKRateCmd *obj = ( CsoundKRateCmd * ) clientData;

	delete obj->_edit;
	obj->_popupRealised = False;
}
