/*
 DialogManager.h - support cached dialog widgets
 modified by Will 18, 2000; will@opaline.demon.co.uk
 
 Adapted from
 Douglas A. Young, 'Object-oriented Programming with C++ and OSF/Motif'
 2nd ed., Prentice-Hall
*/


#ifndef DIALOGMANAGER_H
#define DIALOGMANAGER_H

#include "Interface.h"
#include "DialogCallbackData.h"


class DialogManager : public Interface
{
	public:
		DialogManager ( const char * );
		virtual Widget post ( const char *, void *clientData = NULL,
					  DialogCallback ok = NULL,
					  DialogCallback cancel = NULL,
					  DialogCallback help = NULL );
							  
	protected:
		virtual Widget createDialog ( Widget ) = 0;
		
	private:
		Widget getDialog();
		static void actionCallback ( Widget, XtPointer, XtPointer );
};

#endif

/*
  Notes:

  This is an abstract class; derived classes create actual dialog widgets.

  This class defines the protocol followed by all DialogManager classes. All
  support ok and help, many support cancel. Applications provide a function of
  type DialogCallback.

  Applications needing to display a dialog call the post() member function,
  providing a string to be displayed and optional callback functions for the
  buttons on the dialog.
  
  All derived classes must implement a createDialog() member function which must
  create and return a dialog widget of the type supported by the derived class;
  this widget must be of type XmMessageBox or a subclass of it.
  
  Added a trap for WM protocols - no actions implemented.
*/
