/*
  LineEdit.h 

  Will Eighteen, 2000: will@opaline.demon.co.uk             

  This program is free software; you can redistribute it and/or modify       
  it under the terms of the GNU General Public License as published by       
  the Free Software Foundation; either version 2 of the License, or          
  (at your option) any later version.                                        

  This program is distributed in the hope that it will be useful,            
  but WITHOUT ANY WARRANTY; without even the implied warranty of             
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              
  GNU General Public License for more details.                               

  You should have received a copy of the GNU General Public License          
  along with this program; if not, write to the Free Software                
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                  

  See COPYING for full details.                                  
*/


#ifndef LINEEDIT_H
#define LINEEDIT_H

#include "NoUndoCmd.h"
#include <Xm/Xm.h>


typedef void ( *TextLineCallback ) ( void *, char * );
typedef void ( *TextLineCancelledCallback ) ( void * );


class LineEdit : public NoUndoCmd
{
	public:
		LineEdit ( char *, int, TextLineCallback, TextLineCancelledCallback, void * );
		void setColumns ( int );
		void setText ( char * );
		void setHelp ( char *text ) { ( strncpy ( _helpText, text, sizeof ( _helpText ) ) ); }
		void setEditStatus ( Boolean status );
		void close();
		
	protected:
		void doCommand();		// called by base class
		TextLineCallback _callback;	// function to be called
		TextLineCancelledCallback _cancelCallback;
		void *_clientData;		// data provided by caller
		
	private:
		// callback for WM 'X'
		static void clientMessageProc ( Widget, XtPointer, XtPointer );

		static void enterCallback ( Widget, XtPointer, XtPointer );
		static void okCallback ( Widget, XtPointer, XtPointer );
		static void clearCallback ( Widget, XtPointer, XtPointer );
		static void helpCallback ( Widget, XtPointer, XtPointer );
		static void cancelCallback ( Widget, XtPointer, XtPointer );
		void clientMessage ( XmAnyCallbackStruct * );
		void enter ( char * );
		void clear();
		void help( void * );
		void cancel();
		char _name[128];
		char _applicationName[128];
		char _helpText[256];
		char _setText[256];
		int _columns;
		int _presetText;
		Widget _text;
		Widget _popup;
		Widget _rc1, _rc2, _ok, _clear, _help, _cancel;
		Widget _label;
};

#endif


/*
  Notes:
  
  Pinched from FileSelect ('SelectFileCmd') by Douglas. A. Young. All errors
  due to misunderstanding are entirely mine.
  
  The constructor passes its name and active args to its base class, storing the
  callback and client data for later use.
  
  Usage:
  
  	Cmd *edit = new LineEdit ( "EdiText", TRUE,
								&AppWindow::checkTextCallback,
								( void * ) this );	

	CmdList *cmdListEdit = new CmdList ( "Editor" );
	cmdListEdit->add ( edit );
	_menuBar->addCommands ( cmdListEdit );

  or
  
  	LineEdit *edit = new LineEdit ( "EdiText", TRUE,
								&AppWindow::checkTextCallback,
								( void * ) this );	
	edit->setColumns ( strlen ( "some text" );
	edit->execute();
	edit->setText ( "some text" );	// test display

  for example.
  
  
   Problems:
   
   buffer overflow capabilities ( numerous );
   widget destruction enabled in clear and help callbacks removes buttons;
   as things are, they aren't destroyed, possibly leading to memory leakage.
*/
